_engine = None
_session_mkr = None


def set_global_engine(engine) -> None:
    """Set a global SQLAlchemy engine to avoid passing engine to each decorator

    :param engine: SQLAlchemy engine to set
    """
    global _engine
    _engine = engine


def set_global_sessionmaker(sessionmaker) -> None:
    """Set a global SQLAlchemy session maker to avoid passing it to each decorator

    :param engine: SQLAlchemy session maker to set
    """
    global _session_mkr
    _session_mkr = sessionmaker



from functools import wraps

from sa_decor import globals as G


class with_connection:
    def __init__(self, *, engine=None):
        """Decorate a function and declare it with a named only parameter `connection`
        `def foo(a, b, c=False, *, connection):`
        Then call the function either with an existing connection to pass it to the function
        `foo(1, 2, session=my_connection)`
        Or let the decorator create one for you
        `foo(1, 2)`

        :param engine: Pass an engine if the global is not set (e.g. when using multiple engines in the application)
        """
        self._engine = None
        self._supplied_engine = engine

    def __call__(self, func):
        @wraps(func)
        def session_manager(*args, **kwargs):
            if kwargs.get("connection", None):
                return func(*args, **kwargs)
            with self.engine.connect() as conn:
                return func(*args, **kwargs, connection=conn)

        return session_manager

    @property
    def engine(self):
        if self._engine is None:
            if self._supplied_engine is None and G._engine is None:
                raise ValueError("specify engine or set_global_engine")
            self._engine = (
                self._supplied_engine
                if self._supplied_engine is not None
                else G._engine
            )
        return self._engine



from functools import wraps

from sa_decor import globals as G


class with_session:
    def __init__(
        self, *, commit: bool = None, force_commit: bool = None, session_maker=None
    ):
        """Decorate a function and declare it with a named only parameter `session`
        `def foo(a, b, c=False, *, session):`
        Then call the function either with an existing session to pass it to the function
        `foo(1, 2, session=my_session)`
        Or let the decorator create one for you
        `foo(1, 2)`

        :param commit: Commit the session after the function
        :param force_commit: Commit even when an exception occurs
        :param session_maker: Pass a session maker if the global is not set (e.g. when using multiple session makers in the application)
        :raises ValueError: If the combination of arguments is invalid
        """
        if commit is None and force_commit is None:
            raise ValueError("commit or force_commit must be specified")
        if commit is False and force_commit:
            raise ValueError("cannot force commit and not commit")

        self.commit = commit
        self.force_commit = force_commit
        self._session_maker = None
        self._supplied_session_maker = session_maker

    def __call__(self, func):
        @wraps(func)
        def session_manager(*args, **kwargs):
            if kwargs.get("session", None):
                return func(*args, **kwargs)
            with self.session_maker() as sess:
                error = False
                try:
                    result = func(*args, **kwargs, session=sess)
                    if self.commit:
                        sess.commit()
                    return result
                except Exception:
                    error = True
                    raise
                finally:
                    if self.force_commit and error:
                        sess.commit()

        return session_manager

    @property
    def session_maker(self):
        if self._session_maker is None:
            if self._supplied_session_maker is None and G._session_mkr is None:
                raise ValueError("specify session_maker or set_global_session_maker")
            self._session_maker = (
                self._supplied_session_maker
                if self._supplied_session_maker is not None
                else G._session_mkr
            )
        return self._session_maker



import pytest
from _db_mock.async_engine_queries import create_person, get_persons

from sa_decor import set_global_engine


@pytest.fixture(scope="module", autouse=True)
def set_up_async(async_engine):
    set_global_engine(async_engine)


@pytest.mark.asyncio
async def test_async_no_connection():
    r = await create_person("Karel Barel", 35)
    assert r.rowcount == 1


@pytest.mark.asyncio
async def test_async_existing_connection(async_engine):
    async with async_engine.connect() as conn:
        await create_person("Jean Tomato", 27, connection=conn)
        await create_person("Karel Tomato", 21, connection=conn)
        r = await get_persons("Tomato", connection=conn)
        assert len(r) == 2



import pytest
from _db_mock.session_queries import (
    create_person,
    create_person_with_err,
    create_person_with_err_non_commit,
    get_persons,
)

from sa_decor import set_global_sessionmaker


@pytest.fixture(scope="module", autouse=True)
def set_up_async(session_maker):
    set_global_sessionmaker(session_maker)


def test_no_session():
    r = create_person("Filip C", 24)
    assert r.rowcount == 1


def test_existing_session(session_maker):
    with session_maker() as sess:
        create_person("Jean Tomato", 27, session=sess)
        create_person("Karel Tomato", 21, session=sess)
        r = get_persons("Tomato", session=sess)
        assert len(r) == 2


def test_force_commit_enabled():
    try:
        create_person_with_err("Karel Barel", 28)
    except Exception:
        pass
    r = get_persons("Barel")
    assert len(r) > 0


def test_force_commit_disabled():
    try:
        create_person_with_err_non_commit("Karel Varel", 29)
    except Exception:
        pass
    r = get_persons("Varel")
    assert len(r) == 0



import pytest
from _db_mock.engine_queries import create_person, get_persons

from sa_decor import set_global_engine


@pytest.fixture(scope="module", autouse=True)
def set_up_async(engine):
    set_global_engine(engine)


def test_no_connection():
    r = create_person("Filip C", 24)
    assert r.rowcount == 1


def test_existing_connection(engine):
    with engine.connect() as conn:
        create_person("Jean Tomato", 27, connection=conn)
        create_person("Karel Tomato", 21, connection=conn)
        r = get_persons("Tomato", connection=conn)
        assert len(r) == 2



import os

import setuptools


def get_version() -> str:
    return os.getenv("GITHUB_REF_NAME")[1:]


with open("README.md", "r", encoding="utf-8") as f:
    long_description = f.read()

setuptools.setup(
    name="sa_decor",
    version=get_version(),
    author="Filip Uhlík",
    author_email="filipfilauhlik@gmail.com",
    description="SQLAlchemy decorators for an optional connection/session dependency injection.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/uhlikfil/sa_decor",
    project_urls={
        "Bug Tracker": "https://github.com/uhlikfil/sa_decor/issues",
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    package_dir={"": "./"},
    packages=("sa_decor",),
    python_requires=">=3.7",
)



import pytest
from _db_mock.async_session_queries import (
    create_person,
    create_person_with_err,
    create_person_with_err_non_commit,
    get_persons,
)

from sa_decor import set_global_sessionmaker


@pytest.fixture(scope="module", autouse=True)
def set_up_async(async_session_maker):
    set_global_sessionmaker(async_session_maker)


@pytest.mark.asyncio
async def test_async_no_session():
    r = await create_person("Karel Barel", 35)
    assert r.rowcount == 1


@pytest.mark.asyncio
async def test_async_existing_session(async_session_maker):
    async with async_session_maker() as sess:
        await create_person("Jean Tomato", 27, session=sess)
        await create_person("Karel Tomato", 21, session=sess)
        r = await get_persons("Tomato", session=sess)
        assert len(r) == 2


@pytest.mark.asyncio
async def test_force_commit_enabled():
    try:
        await create_person_with_err("Karel Barel", 28)
    except Exception:
        pass
    r = await get_persons("Barel")
    assert len(r) > 0


@pytest.mark.asyncio
async def test_force_commit_disabled():
    try:
        await create_person_with_err_non_commit("Karel Varel", 29)
    except Exception:
        pass
    r = await get_persons("Varel")
    assert len(r) == 0
